package java.security.cert;

import java.security.AccessController;
import java.security.Security;
import java.security.PrivilegedAction;
import java.security.NoSuchProviderException;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import sun.security.jca.GetInstance;
import java.util.Collection;
import java.security.Provider;

public class CertStore
{
    private static final String CERTSTORE_TYPE = "certstore.type";
    private CertStoreSpi storeSpi;
    private Provider provider;
    private String type;
    private CertStoreParameters params;
    
    protected CertStore(CertStoreSpi storeSpi, Provider provider, String type, CertStoreParameters certStoreParameters) {
        this.storeSpi = storeSpi;
        this.provider = provider;
        this.type = type;
        if (certStoreParameters != null) {
            this.params = (CertStoreParameters)certStoreParameters.clone();
        }
    }
    
    public final Collection<? extends Certificate> getCertificates(CertSelector certSelector) throws CertStoreException {
        return this.storeSpi.engineGetCertificates(certSelector);
    }
    
    public final Collection<? extends CRL> getCRLs(CRLSelector crlSelector) throws CertStoreException {
        return this.storeSpi.engineGetCRLs(crlSelector);
    }
    
    public static CertStore getInstance(String algorithm, CertStoreParameters param) throws InvalidAlgorithmParameterException, NoSuchAlgorithmException {
        try {
            GetInstance.Instance instance = GetInstance.getInstance("CertStore", CertStoreSpi.class, algorithm, param);
            return new CertStore((CertStoreSpi)instance.impl, instance.provider, algorithm, param);
        }
        catch (NoSuchAlgorithmException ex) {
            return handleException(ex);
        }
    }
    
    private static CertStore handleException(NoSuchAlgorithmException ex) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException {
        Throwable cause = ex.getCause();
        if ((cause instanceof InvalidAlgorithmParameterException)) {
            throw (InvalidAlgorithmParameterException)cause;
        }
        throw ex;
    }
    
    public static CertStore getInstance(String algorithm, CertStoreParameters param, String provider) throws InvalidAlgorithmParameterException, NoSuchAlgorithmException, NoSuchProviderException {
        try {
            GetInstance.Instance instance = GetInstance.getInstance("CertStore", CertStoreSpi.class, algorithm, param, provider);
            return new CertStore((CertStoreSpi)instance.impl, instance.provider, algorithm, param);
        }
        catch (NoSuchAlgorithmException ex) {
            return handleException(ex);
        }
    }
    
    public static CertStore getInstance(String algorithm, CertStoreParameters param, Provider provider) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException {
        try {
            GetInstance.Instance instance = GetInstance.getInstance("CertStore", CertStoreSpi.class, algorithm, param, provider);
            return new CertStore((CertStoreSpi)instance.impl, instance.provider, algorithm, param);
        }
        catch (NoSuchAlgorithmException ex) {
            return handleException(ex);
        }
    }
    
    public final CertStoreParameters getCertStoreParameters() {
        return (this.params == null) ? null : ((CertStoreParameters)this.params.clone());
    }
    
    public final String getType() {
        return this.type;
    }
    
    public final Provider getProvider() {
        return this.provider;
    }
    
    public static final String getDefaultType() {
        String s = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return Security.getProperty("certstore.type");
            }
        });
        if (s == null) {
            s = "LDAP";
            return s;
        }
        return s;
    }
}
